@@ -1,27 +1,32 @@
-# from bubot.OcfDriver.OcfDevice.OcfDevice import OcfDevice
-from BubotObj.OcfDevice.subtype.VirtualServer.VirtualServer import VirtualServer
-import json
-from bson import ObjectId
-from BubotObj.OcfDevice.subtype.Device.QueueMixin import QueueMixin
-from .__init__ import __version__ as device_version
-from aiohttp import web
-from aiohttp_session import get_session, setup
-from BubotObj.OcfDevice.subtype.WebServer.AppSessionStorage import AppSessionStorage
+# from bubot.Client.OcfDevice.OcfDevice import OcfDevice
 import asyncio
-# import logging
-from BubotObj.OcfDevice.subtype.WebServer.HttpHandler import HttpHandler
-from BubotObj.OcfDevice.subtype.WebServer.FormHandler import FormHandler
-from BubotObj.OcfDevice.subtype.WebServer.WsHandler import WsHandler
+import json
+import os.path
+from uuid import uuid4
+import inspect
 
-# from bubot.Catalog.OcfDriver.WebServer import API
+from aiohttp import web
+from aiohttp_session import get_session, setup, session_middleware
+# from bson import ObjectId
+
+# from bubot.Catalog.Client.WebServer import API
 from Bubot.Core.DataBase.Mongo import Mongo as Storage
 # from bubot.Core.DataBase.SqlLite import SqlLite as Storage
-from Bubot.Core.FastStorage.PythonFastStorage import PythonFastStorage as FastStorage
-from Bubot.Helpers.Helper import Helper
+from Bubot.Core.FastStorage.Simple import SimpleFastStorage as FastStorage
 from Bubot.Helpers.ActionDecorator import async_action
 from Bubot.Helpers.ExtException import ExtException, ResourceNotAvailable
+from Bubot.Helpers.Helper import Helper
 from Bubot.Ocf.Helper import find_drivers
-import os.path
+from BubotObj.OcfDevice.subtype.Device.Device import Device
+from BubotObj.OcfDevice.subtype.Device.QueueMixin import QueueMixin
+from BubotObj.OcfDevice.subtype.VirtualServer.VirtualServer import VirtualServer
+from BubotObj.OcfDevice.subtype.WebServer.AppSessionStorage import AppSessionStorage
+from BubotObj.OcfDevice.subtype.WebServer.FormHandler import FormHandler
+# import logging
+from BubotObj.OcfDevice.subtype.WebServer.HttpHandler import HttpHandler, PublicHttpHandler
+from BubotObj.OcfDevice.subtype.WebServer.WsHandler import WsHandler
+from .__init__ import __version__ as device_version
+import re
 
 
 # _logger = logging.getLogger(__name__)
@@ -43,6 +48,7 @@ class WebServer(VirtualServer, QueueMixin):
         self.serial_queue_worker = None
         self.ws = {}
         self.runner = None
+        self.storage = None
         VirtualServer.__init__(self, **kwargs)
 
     async def on_pending(self):
@@ -50,6 +56,13 @@ class WebServer(VirtualServer, QueueMixin):
         await self.run_web_server()
         await super().on_pending()
 
+    async def on_cancelled(self):
+        if self.storage:
+            await self.storage.close()
+        if self.runner is not None:
+            await self.runner.cleanup()
+        await super().on_cancelled()
+
     @async_action
     async def run_web_server(self, *, _action):
 
@@ -57,21 +70,24 @@ class WebServer(VirtualServer, QueueMixin):
         # self.save_config()
         # self.log.info(f'{self.__class__.__name__} start up')
         app = web.Application(
-            middlewares=[
-                # session_middleware(SimpleCookieStorage()),
-                # self.middleware_auth(),
-                self.middleware_index
-            ])
+            # middlewares=[
+            #     self.middleware_auth,
+            #     self.middleware_index
+            # ]
+        )
         app['device'] = self
         app['sessions'] = {}
         app['fast_storage'] = FastStorage()
-        app['storage'] = Storage.connect(self)
+        self.storage = await Storage.connect(self)
+        app['storage'] = self.storage
+        app.middlewares.append(self.middleware_index)
         setup(app, self.get_session_storage(app, 'AppSessionStorage'))
+        app.middlewares.append(self.middleware_auth)
         drivers = find_drivers(log=self.log)
         self.set_param('/oic/mnt', 'drivers', drivers)
         self.build_i18n(drivers)
         self.add_routes(app)
-        port = self.get_param('/oic/con', 'port', 80)
+        port = self.get_param('/oic/con', 'port', 8080)
         # app.on_startup.append(self.start_background_tasks)
         # app.on_cleanup.append(self.cleanup_background_tasks)
         self.runner = web.AppRunner(app)
@@ -116,8 +132,12 @@ class WebServer(VirtualServer, QueueMixin):
                 with open(locale_path, "r", encoding='utf-8') as file:
                     try:
                         _data = json.load(file)
-                        Helper.update_dict(locales[locale], _data)
+                        if isinstance(_data, dict):
+                            Helper.update_dict(locales[locale], _data)
+                        else:
+                            self.log.error(f'Build locale {locale} for driver {elem}: Bad format {_data}')
                     except Exception as err:
+                        err = ExtException(parent=err)
                         self.log.error(f'Build locale {locale} for driver {elem}: {str(err)}')
 
         i18n_dir = os.path.normpath(f'{self.path}/i18n')
@@ -135,56 +155,60 @@ class WebServer(VirtualServer, QueueMixin):
                 except Exception as err:
                     self.log.error(f'Build locale {locale}: {str(err)}')
 
-        self.log.info('complete')
+        self.log.info('build i18n complete')
 
     def add_routes(self, app):
-        i = 0
         self.log.info('add routes')
         for elem in self.get_param('/oic/mnt', 'drivers'):
             try:
-                ui_view = self.get_device_class(elem)()
-                if hasattr(ui_view, 'add_route'):
-                    ui_view.add_route(app)  # todo сделать разводящую из всех доступных
-                    i += 1
+                ui_view: Device = self.get_device_class(elem)(path=self.path)
+                ui_view.add_route(app)  # todo сделать разводящую из всех доступных
+            except NotImplementedError:
+                pass
             except Exception as e:
-                self.log.error('Error import_ui_handlers({1}): {0}'.format(e, elem))
-
+                err = ExtException(parent=e)
+                self.log.error(f'Error import_ui_handlers({elem}): {err}')
+        # for elem in app.router.routes():
+        #     print(elem)
         pass
 
     def add_route(self, app):
         app.router.add_route('get', '/ws', WsHandler)
         app.router.add_route('*', '/api/{device}/{action}', HttpHandler)
         app.router.add_route('*', '/api/{device}/{obj_name}/{action}', HttpHandler)
+        app.router.add_route('*', '/api/{device}/{obj_name}/{subtype}/{action}', HttpHandler)
+        app.router.add_route('*', '/public_api/{device}/{action}', PublicHttpHandler)
+        app.router.add_route('*', '/public_api/{device}/{obj_name}/{action}', PublicHttpHandler)
         app.router.add_route('get', '/form/{device}/{obj_name}/{form_name}', FormHandler)
+        app.router.add_route('get', '/form/{device}/{obj_name}/{subtype}/{form_name}', FormHandler)
+        # app.router.add_route('*', '/schema/{action}', SchemaHandler)
         app.router.add_static('/i18n', f'{self.path}/i18n')
-
         pass
 
     @staticmethod
-    def middleware_auth():
-        async def middleware_factory(app, handler):
-            async def auth_handler(request):
-                session = await get_session(request)
-                if session.get("user"):
-                    return await handler(request)
-                else:
-                    # auth = 0
-                    try:
-                        pass
-                        # if issubclass(handler, Ui) and handler.need_auth(request):
-                        #     raise web.HTTPFound(
-                        #         "{0}?redirect={1}".format(app['bubot'].get_param('login_url'), request.path))
-                        # auth = request.app['ui'][re.findall('^/ui/(.*)/', request.path)[0]]['param']['auth']
-                    finally:
-                        pass
-                        # if auth:
-                        # url = request.app.router['login'].url()
+    @web.middleware
+    async def middleware_auth(request, handler):
+        try:
+            session = await get_session(request)
+        except Exception as err:
+            raise err
+        if session.get("user"):
+            return await handler(request)
+        else:
+            # auth = 0
+            # try:
+            if handler in [HttpHandler, WsHandler]:
+                raise web.HTTPUnauthorized()
+                # url = request.app['device'].get_param('/oic/con', 'login_url', '/ui/AuthService')
+                # redirect_url = request.path
+                # raise web.HTTPFound(f"{url}?redirect={redirect_url}")
+                # auth = request.app['src_vue'][re.findall('^/src_vue/(.*)/', request.path)[0]]['param']['auth']
+            # finally:
+            #     pass
+            # if auth:
+            #     url = request.app.router['login'].url()
 
-                return await handler(request)
-
-            return auth_handler
-
-        return middleware_factory
+        return await handler(request)
 
     @staticmethod
     @web.middleware
@@ -197,6 +221,7 @@ class WebServer(VirtualServer, QueueMixin):
         # """
 
         try:
+
             filename = request.match_info['filename']
             if not filename or filename.endswith('/'):
                 filename = index
@@ -234,7 +259,7 @@ class WebServer(VirtualServer, QueueMixin):
             return AppSessionStorage(
                 app,
                 httponly=False,
-                key_factory=lambda: str(ObjectId()),
+                key_factory=lambda: str(uuid4()),
                 cookie_name="session",
                 # encoder=cookie_encoder, decoder=cookie_decoder
             )
